import json
import yaml

class Logger():
    def __init__(self):
        self.logs = []
        self.invalid_path_errors = []
        self.warnings = 0
        self.errors = 0
        self.severe = 0
        self.output_file_name = None
        self.output_fmt = "json"
    
    def set_file_name_and_format(self, file_name, output_fmt):
        self.output_file_name = file_name
        self.output_fmt = output_fmt
    
    def log_with_line_number(self, num_line, severity, message, id):
        tmp_dict = {
            "id": id,
            "num_line": num_line,
            "severity": severity,
            "message": message
        }
        self.logs.append(tmp_dict)
        if severity == "WARN":
            self.warnings += 1
        else:
            self.errors += 1

    def log_with_line_number_key_and_value(self, num_line, severity, message, id, key, value):
        tmp_dict = {
            "id": id,
            "num_line": num_line,
            "severity": severity,
            "message": message,
            "key": key,
            "value": value
        }
        self.logs.append(tmp_dict)
        if severity == "WARN":
            self.warnings += 1
        else:
            self.errors += 1
    
    def log_with_line_number_key(self, num_line, severity, message, id, key):
        tmp_dict = {
            "id": id,
            "num_line": num_line,
            "severity": severity,
            "message": message,
            "key": key
        }
        self.logs.append(tmp_dict)
        if severity == "WARN":
            self.warnings += 1
        else:
            self.errors += 1
        
    def log_with_property(self, property_, severity, message, id):
        tmp_dict = {
            "id": id,
            "property": property_,
            "severity": severity,
            "message": message
        }
        self.logs.append(tmp_dict)
        if severity == "WARN":
            self.warnings += 1
        else:
            self.errors += 1
    
    def log_internal_references(self, num_line, message, key, value):
        tmp_dict = {
            "num_line": num_line,
            "severity": "ERROR",
            "message": message,
            "key": key,
            "value": value
        }
        self.logs.append(tmp_dict)
        self.errors += 1
    
    def log_invalid_path(self, line_num, message):
        tmp_dict = {
            "severity": "SEVERE",
            "line_num": line_num,
            "message": message,
        }
        self.invalid_path_errors.append(tmp_dict)
        self.severe += 1

    def get_logs_json(self):
        if self.severe > 0:
            return json.dumps(
                {
                    "total": self.severe,
                    "logs": self.invalid_path_errors
                },
                indent = 4
            )
        else:
            return json.dumps(
                {
                    "total": len(self.logs),
                    "warnings": self.warnings,
                    "errors": self.errors,
                    "logs":self.logs
                }, 
                    indent=4
                )

    def get_logs_yaml(self):
        if self.severe > 0:
            return yaml.dump(
                {
                    "total": self.severe,
                    "logs": self.invalid_path_errors
                },
                indent = 4,
                sort_keys = False
            )
        else:
            return yaml.dump(
                {
                    "total": len(self.logs),
                    "warnings": self.warnings,
                    "errors": self.errors,
                    "logs": self.logs
                },
                    indent=4
                , sort_keys = False
                )
    
    def print_and_save_logs(self):
        with open(self.output_file_name+f"-output.{self.output_fmt}", "w") as of:
            if self.output_fmt=="json":
                print(self.get_logs_json())
                of.write(self.get_logs_json())
            elif self.output_fmt == "yaml":
                print(self.get_logs_yaml())
                of.write(self.get_logs_yaml())